@@ -1,42 +1,18 @@
-"""
-    Copyright (C) 2017, ContraxSuite, LLC
-
-    This program is free software: you can redistribute it and/or modify
-    it under the terms of the GNU Affero General Public License as
-    published by the Free Software Foundation, either version 3 of the
-    License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Affero General Public License for more details.
-
-    You should have received a copy of the GNU Affero General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-    You can also be released from the requirements of the license by purchasing
-    a commercial license from ContraxSuite, LLC. Buying such a license is
-    mandatory as soon as you develop commercial activities involving ContraxSuite
-    software without disclosing the source code of your own applications.  These
-    activities include: offering paid services to customers as an ASP or "cloud"
-    provider, processing documents on the fly in a web application,
-    or shipping ContraxSuite within a closed source product.
-"""
-
 # -*- coding: utf-8 -*-
 
+__author__ = "ContraxSuite, LLC; LexPredict, LLC"
+__copyright__ = "Copyright 2015-2021, ContraxSuite, LLC"
+__license__ = "https://github.com/LexPredict/lexpredict-lexnlp/blob/2.1.0/LICENSE"
+__version__ = "2.1.0"
+__maintainer__ = "LexPredict, LLC"
+__email__ = "support@contraxsuite.com"
+
+
 from typing import List, Optional
 
 import regex as re
 import pandas as pd
 
-__author__ = "ContraxSuite, LLC; LexPredict, LLC"
-__copyright__ = "Copyright 2015-2020, ContraxSuite, LLC"
-__license__ = "https://github.com/LexPredict/lexpredict-lexnlp/blob/1.8.0/LICENSE"
-__version__ = "1.8.0"
-__maintainer__ = "LexPredict, LLC"
-__email__ = "support@contraxsuite.com"
-
 
 class EntityBanListItem:
     def __init__(self,
@@ -91,7 +67,7 @@ class EntityBanListItem:
         if self.trim_phrase:
             text = text.strip()
         if self.reg_exp:
-            return True if self.reg_exp.match(text) else False
+            return self.reg_exp.match(text) is not None
         if self.ignore_case:
             text = (text or '').lower()
         return self.pattern == text
@@ -117,11 +93,11 @@ class EntityBanListItem:
             ignore_case, is_regex, trim_phrase = True, False, True
             columns = len(row)
             if columns > 1:
-                ignore_case = True if row[1] else False
+                ignore_case = bool(row[1])
             if columns > 2:
-                is_regex = True if row[2] else False
+                is_regex = bool(row[2])
             if columns > 3:
-                trim_phrase = True if row[3] else False
+                trim_phrase = bool(row[3])
 
             item = EntityBanListItem(pattern, ignore_case, is_regex, trim_phrase)
             items.append(item)
